@@ -1,5 +1,5 @@
 # -*-Perl-*-
-# @(#)Sybperl.pm	1.20	9/30/95
+# %W%	%G%
 #
 # Copyright (c) 1994-1995
 #   Michael Peppler
@@ -39,10 +39,10 @@ package Sybase::Sybperl;
 
 use Carp;
 require Exporter;
-require AutoLoader;
+use AutoLoader 'AUTOLOAD';
 use Sybase::DBlib;
 
-@ISA = qw(Exporter AutoLoader);
+@ISA = qw(Exporter);
 
 $SUCCEED = Sybase::DBlib::SUCCEED;
 $FAIL = Sybase::DBlib::FAIL;
@@ -55,6 +55,7 @@ $DBFALSE = Sybase::DBlib::FALSE;
 $DB_IN = Sybase::DBlib::DB_IN;
 $DB_OUT = Sybase::DBlib::DB_OUT;
 
+$VERSION = $Sybase::DBlib::VERSION;
 # Set defaults.
 tie $dbNullIsUndef, Sybase::Sybperl::Attribs, 'dbNullIsUndef', 1;
 tie $dbKeepNumeric, Sybase::Sybperl::Attribs, 'dbKeepNumeric', 1;
@@ -65,7 +66,7 @@ tie $dbBin0x, Sybase::Sybperl::Attribs, 'dbBin0x', 0;
 @EXPORT = qw(dblogin dbcmd dbsqlexec dbresults dbnextrow dbstrcpy
 	     dbuse dbopen dbclose
 	     $SUCCEED $FAIL $NO_MORE_RESULTS $NO_MORE_ROWS
-	     $MORE_ROWS $REG_ROW $DBTRUE $DBFALSE
+	     $MORE_ROWS $REG_ROW $DBTRUE $DBFALSE $DB_IN $DB_OUT
 	     $dbNullIsUndef $dbKeepNumeric $dbBin0x
 	     bcp_init bcp_meminit bcp_sendrow bcp_batch bcp_done
 	     bcp_control bcp_columns bcp_colfmt bcp_collen bcp_exec
@@ -90,7 +91,8 @@ tie $dbBin0x, Sybase::Sybperl::Attribs, 'dbBin0x', 0;
 	     dbmnymul dbmnyminus dbmnydivide dbmnyadd dbmny4zero
 	     dbmny4cmp dbmny4sub dbmny4mul dbmny4minus dbmny4divide
 	     dbmny4add sql
-	     dbrpcinit dbrpcparam dbrpcsend);
+	     dbrpcinit dbrpcparam dbrpcsend
+	     dbwritetext dbreadtext dbmoretext dbpreptext);
 
 
 # Internal routine to check that a parameter passed as $dbproc to one
@@ -147,8 +149,12 @@ sub dbclose
     croak "&dbclose() must be called with an argument!\n"
 	if(!defined($dbproc) || !&isadb($dbproc));
 
+    if($default_db == $dbproc) {
+	undef($default_db);
+    }
+
     delete($DBprocs{$dbproc});
-    $dbproc->dbclose;
+    $dbproc->force_dbclose;
     undef($dbproc);
 }
 
@@ -214,6 +220,20 @@ sub dbsqlexec
     $ret;
 }
 
+sub dbsqlok
+{
+    my($dbproc) = @_;
+    my($ret);
+
+    if(!defined($dbproc) || !$dbproc || !&isadb($dbproc))
+    {
+	croak("It doesn't make sense to call dbsqlok with an undefined \$dbproc") if(!defined($default_db));
+	$dbproc = $default_db;
+    }
+    $ret = $dbproc->dbsqlok;
+    $ret;
+}
+
 sub dbresults
 {
     my($dbproc) = @_;
@@ -243,7 +263,7 @@ sub dbnextrow
     
     @row = $dbproc->dbnextrow(@params);
     
-    $main::ComputeID = $dbproc->{'ComputeID'};
+    $main::ComputeId = $dbproc->{'ComputeID'};
     $main::DBstatus = $dbproc->{'DBstatus'};
 
     @row;
@@ -324,6 +344,28 @@ sub dbfreebuf
     $dbproc->dbfreebuf;
 }
 
+sub dbsetopt
+{
+    my($dbproc, @param) = @_;
+
+    $dbproc->dbsetopt(@param);
+}
+
+sub dbclropt
+{
+    my($dbproc, @param) = @_;
+
+    $dbproc->dbclropt(@param);
+}
+
+sub dbisopt
+{
+    my($dbproc, @param) = @_;
+
+    $dbproc->dbisopt(@param);
+}
+
+
 sub DBCURCMD
 {
     my($dbproc) = @_;
@@ -1086,4 +1128,47 @@ sub dbrpcinit
     $dbproc->dbrpcinit(@param);
 }
 
+sub dbwritetext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbwritetext(@params);
+}
+
+sub dbreadtext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbwritetext(@params);
+}
+
+sub dbmoretext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbmoretext(@params);
+}
+
+sub dbpreptext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbpreptext(@params);
+}
+
+
 1;
+
+
